/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-09-12
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
*/

package spirals.data;

import java.awt.Graphics;
import javax.swing.JPanel;

/**
 * <p>Abstract class providing method for viewing captured data</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.1
 */
public abstract class TabletDataViewer {
  /**
   * captured data
   */
  TabletData tabletData;
  /**
   * panel to draw on
   */
  JPanel screen;

  /**
   * creates new viewer
   * @param screen panel to draw on
   * @param tabletData captured data from tablet
   */
  public TabletDataViewer(JPanel screen, TabletData tabletData) {
    this.tabletData = tabletData;
    this.screen = screen;
  }

  /**
   * derived class must override this method to implement drawing
   */
  public abstract void paint(Graphics g);

  /**
   * repaint provided panel with data
   */
  public final void repaint() {
    this.paint(screen.getGraphics());
  }

  /**
   * provides new TabletData for viewing
   */
  public final void setTabletData(TabletData tabletData) {
    this.tabletData = tabletData;
  }
}
