/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-11-30
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.database;

import spirals.motol.database.Configuration;
import org.jdom.*;

/**
 * <p>Class contains information about one test performed on tablet</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.3
 */
public class Test {
  /**
   * name of type tag id attribute
   */
  public static final String typeIdAttr = "id";
  /**
   * name of type tag name attribute
   */
  public static final String typeNameAttr = "name";
  /**
   * name of type tag start attribute
   */
  public static final String typeStartAttr = "start";
  /**
   * name of type tag start attribute default value
   */
  public static final String startAttrValue = "start";
  /**
   * name of type tag end attribute
   */
  public static final String typeEndAttr = "end";
  /**
   * name of type tag end attribute default value
   */
  public static final String endAttrValue = "trim";
  /**
   * name of type tag visible attribute
   */
  public static final String typeVisibleAttr = "visible";
  /**
   * name of type tag visible attribute default value
   */
  public static final String visibleAttrValue = "on";
  /**
   * name of type tag show attribute
   */
  public static final String typeShowAttr = "show";
  /**
   * name of type tag show attribute default value
   */
  public static final String showAttrValue = "up";

  private int id;
  private String name;
  private boolean start;
  private boolean trim;
  private boolean visible;
  private boolean show;

  public Test(Element e) {
    id = Integer.parseInt(e.getAttributeValue(typeIdAttr));
    name = e.getAttributeValue(typeNameAttr);
    start = (e.getAttributeValue(typeStartAttr).equals(startAttrValue));
    trim = (e.getAttributeValue(typeEndAttr).equals(endAttrValue));
    visible = (e.getAttributeValue(typeVisibleAttr).equals(visibleAttrValue));
    show = (e.getAttributeValue(typeShowAttr).equals(showAttrValue));
  }

  public Element toElement() {
    Element e = new Element(Configuration.typeTag);
    e.setAttribute(typeIdAttr, "" + id);
    e.setAttribute(typeNameAttr, name);
    e.setAttribute(typeStartAttr, start ? startAttrValue : "button");
    e.setAttribute(typeEndAttr, trim ? endAttrValue : "full");
    e.setAttribute(typeVisibleAttr, visible ? visibleAttrValue : "off");
    e.setAttribute(typeShowAttr, visible ? showAttrValue : "down");
    return e;
  }

  public int getId() {
    return id;
  }

  public String getName() {
    return name;
  }

  public void setStart(boolean start) {
    this.start = start;
  }

  public boolean startByStart() {
    return start;
  }

  public void setTrim(boolean trim) {
    this.trim = trim;
  }

  public boolean trim() {
    return trim;
  }

  public void setVisible(boolean show) {
    this.visible = show;
  }

  public boolean isVisible() {
    return visible;
  }

  public boolean showUp() {
    return show;
  }

  public String toString() {
    return name;
  }
}
