/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-11-30
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.database;

import java.util.*;
import spirals.motol.database.*;
/**
 * <p>Class contains information about tests performed by one person</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.3
 */
public class TestList {
  private Visit parent;
  private static List<Test> tests = null;
  private int hist[];

  public TestList(Visit v) {
    parent = v;
    if(tests == null) tests = parent.getPacient().getDatabase().getEngine().getConfiguration().getTests();
    int max = tests.size();
    for(Test t: tests) {
      if(t.getId() >= max) {
        max = t.getId() + 1;
      }
    }
    hist = new int[max];
  }

  public void addHist(String desc) {
    for(Test t: tests) {
      if(t.getName().equals(desc)) {
        hist[t.getId()]++;
        break;
      }
    }
  }

  public int getHist(String desc) {
    for(Test t: tests) {
      if(t.getName().equals(desc)) {
        return hist[t.getId()];
      }
    }
    return -1;
  }

  public void updateHist(String oldName, String newName) {
    for(Test t: tests) {
      if(t.getName().equals(newName)) {
        hist[t.getId()]++;
      }
      if(t.getName().equals(oldName)) {
        hist[t.getId()]--;
      }
    }
  }

  public static Test get(String desc) {
    for(Test t: tests) {
      if(t.getName().equals(desc)) {
        return t;
      }
    }
    return null;
  }

  public static List<String> getShowable() {
    List<String> res = new ArrayList<String>(tests.size());
    for(Test t: tests) {
      if(t.isVisible()) res.add(t.getName());
    }
    return res;
  }

  public static List<String> getAll() {
    List<String> res = new ArrayList<String>(tests.size());
    for(Test t: tests) {
      res.add(t.getName());
    }
    return res;
  }
}
