/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-11-30
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
 */

package spirals.motol.gui;

import spirals.motol.database.*;
import spirals.data.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import tablet.*;
import java.util.*;
import java.text.SimpleDateFormat;
import java.awt.image.*;
import java.io.File;

/**
 * <p>Frame where data from tablet are catured and saved.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.3
 */
public class CaptureFrame
    extends JDialog implements TabletListener {
  /**
   * old x position
   */
  private int ox;
  /**
   * old y position
   */
  private int oy;
  /**
   * tablet pen is touching surface
   */
  private boolean pen_down = false;
  private boolean show;
  protected boolean firstPacket;
  protected long startTime;
  /**
   * tablet data viewer
   */
  private SimpleTabletDataViewer viewer;

  /**
   * visit for which are data captured
   */
  protected Visit visit;
  /**
   * contains contents of frame
   */
  private JPanel contentPane;
  /**
   * actual tablet measures
   */
  private TabletInfo tabletInfo;
  /**
   * picture for already drawn type of data
   */
  private Icon okIcon = new ImageIcon(CaptureFrame.class.getResource(
      "/res/ok.gif"));
  /**
   * engine
   */
  private Engine engine;
  /**
   * capturing tablet data
   */
  private TabletData tabletData;

  /**
   * some tablet data ready
   */
  protected boolean data = false;
  /**
   * button performing start of capturing
   */
  protected JButton startButton;
  /**
   * buttons for saving captured data
   */
  protected JButton saveButton = new JButton(new Capture_Save_Action(this));
  /**
   * button performing zooming in captured data
   */
  private JButton detailButton = new JButton(new AbstractAction("Detail") {
    public void actionPerformed(ActionEvent e) {
      viewer.setDetail(!viewer.getDetail());
      setScreenBorder();
    }
  });

  /**
   * buttons for closing frame
   */
  private JButton exitButton = new JButton(new Capture_Exit_Action(this));
  /**
   * list of types of drawings
   */
  private ButtonGroup type = new ButtonGroup();
  /**
   * container for types of drawings
   */
  private JPanel leftCenter;
  /**
   * data comment
   */
  private JEditorPane memo = new JEditorPane();

  /**
   * area for view drawing
   */
  protected JPanel screen = new JPanel() {
    public Dimension getPreferredSize() {
      return new Dimension(600, 400);
    }

    public void paintComponent(Graphics g) {
      viewer.paint(g);
    }
  };

  protected TimeThread time;

  /**
   * create frame for visit
   */
  public CaptureFrame(Engine engine, Visit visit) {
    super(engine.getMainFrame(), "Zznam dat z tabletu", true);
    this.engine = engine;
    this.visit = visit;
    this.tabletInfo = engine.getTablet().getTabletInfo();
    this.tabletData = new TabletData(tabletInfo);
    this.viewer = new SimpleTabletDataViewer(screen, tabletData, true);

    this.contentPane = (JPanel)this.getContentPane();
    this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    this.addWindowListener(new Capture_Exit_Action(this));
    this.createLayout();
    this.setScreenBorder();
    this.colorize();
  }

  /**
   * create and place frame components
   */
  private void createLayout() {
    JPanel bottom = new JPanel();
    JPanel center = new JPanel(new BorderLayout());
    startButton = new Capture_Start_Button(engine, this);
    bottom.add(detailButton);
    bottom.add(Box.createHorizontalStrut(10));
    bottom.add(startButton);
    bottom.add(Box.createHorizontalStrut(10));
    bottom.add(saveButton);
    saveButton.setEnabled(false);
    detailButton.setEnabled(false);
    memo.setEnabled(false);
    bottom.add(Box.createHorizontalStrut(10));
    bottom.add(exitButton);

    screen.setBorder(BorderFactory.createEtchedBorder());
    center.add(screen, BorderLayout.CENTER);
    center.add(bottom, BorderLayout.SOUTH);

    contentPane.add(createLeftPanel(), BorderLayout.WEST);
    contentPane.add(center, BorderLayout.CENTER);
  }

  /**
   * create and return panel with types of drawings
   */
  private JPanel createLeftPanel() {
    JPanel left = new JPanel(new BorderLayout());
    java.util.List<String> types = visit.getTestList().getShowable();
    leftCenter = new JPanel(new GridLayout(types.size(), 1));
    JPanel bottom = new JPanel(new BorderLayout());
    leftCenter.setBorder(BorderFactory.createEtchedBorder());
    for (int i = 0; i < types.size(); i++) {
      JPanel row = new JPanel(new FlowLayout(FlowLayout.LEFT, 5, 0));
      JRadioButton radio = new JRadioButton(types.get(i).toString());
      radio.setActionCommand(types.get(i).toString());
      type.add(radio);
      row.add(radio);
      row.add(new JLabel());
      leftCenter.add(row);
    }
    bottom.setBorder(BorderFactory.createEtchedBorder());
    bottom.add(new JLabel("Poznmky:"), BorderLayout.NORTH);
    bottom.add(new JScrollPane(memo), BorderLayout.CENTER);
    bottom.setPreferredSize(new Dimension(200, 100));

    left.add(leftCenter, BorderLayout.NORTH);
    left.add(bottom, BorderLayout.CENTER);
    return left;
  }

  public void onTimeout() {
    JOptionPane.showMessageDialog(engine.getMainFrame(),
                                  "Snmn dat pekroilo maximln povolenou dlku " +
                                  engine.getConfiguration().getTimeout() +
                                  " vtein. Snmn bylo ukoneno",
                                  "Chyba",
                                  JOptionPane.ERROR_MESSAGE);
    startButton.setText("Start");
    this.setDataEditEnabled(false);
    this.setButtonsEnabled(true);
    this.tabletData = new TabletData(tabletInfo);
    viewer.setTabletData(this.tabletData);
    screen.repaint();
    this.setRadioEnabled(true);
    data = false;
  }

  /**
   * called by tablet when tablet packet is ready
   * <p>Add packet to data sequence add display new point.</p>
   */
  public void onTabletPacket(TabletPoint point) {
    if (firstPacket) {
      firstPacket = false;
      startTime = System.currentTimeMillis();
      time = new TimeThread(this);
      time.start();
    }
    data = true;
    tabletData.add(point);
    // view and tablet area measures
    Graphics g = screen.getGraphics();
    int scr_width = screen.getWidth();
    int scr_height = screen.getHeight();
    int tab_width = tabletInfo.getWidth();
    int tab_height = tabletInfo.getHeight();

    // which dimension have bigger compress ratio
    float r_w = ( (float) scr_width) / (float) tab_width;
    float r_h = ( (float) scr_height) / (float) tab_height;
    float koef = (r_h > r_w) ? r_w : r_h;

    int x = (int) (point.getX() * koef);
    int y = (int) (scr_height - point.getY() * koef);

    if (!point.isTouched() && show) {
      pen_down = false;
      return;
    }

    if (!pen_down) { // beginning of new line
      pen_down = true;
      ox = x;
      oy = y;
    }

    g.setColor(Color.black);
    g.drawLine(x, y, ox, oy);
    ox = x;
    oy = y;
  }

  /**
   * return selected type of drawing
   */
  protected String getType() {
    return type.getSelection().getActionCommand();
  }

  /**
   * start capturing form tablet
   */
  protected void startCapture() {
    setButtonsEnabled(false);
    setDataEditEnabled(false);
    tabletData = new TabletData(tabletInfo);
    pen_down = false;
    viewer.setDetail(false);
    Test t = visit.getTestList().get(getType());
    viewer.setShowUp(t.showUp());
    viewer.setTabletData(tabletData);
    this.setScreenBorder();
    firstPacket = true;
    show = t.showUp();
    engine.getTablet().startCapture(t.startByStart());

    if (time != null) time.end();
  }

  /**
   * colorize all already captured types of drawings and set next unused
   */
  private void colorize() {
    Component[] left = leftCenter.getComponents();
    TestList tests = visit.getTestList();
    // colorize all used types
    for (int i = 0; i < left.length; i++) {
      Component[] list = ( (JPanel) left[i]).getComponents();
      AbstractButton button = ( (AbstractButton) list[0]);
      int num = tests.getHist(button.getText());
      if (num > 0) {
        button.setForeground(Color.GREEN);
        ( (JLabel) list[1]).setText(" (" + num + ")");
        ( (JLabel) list[1]).setIcon(okIcon);
      }
    }
    // set next unused type
    for (int i = 0; i < left.length; i++) {
      Component[] list = ( (JPanel) left[i]).getComponents();
      AbstractButton button = ( (AbstractButton) list[0]);
      int num = tests.getHist(button.getText());
      if (num == 0) {
        ( (AbstractButton) ( (JPanel) left[i]).getComponents()[0]).setSelected(true);
        return;
      }
    }
  }

  /**
   * save captured data to disk
   */
  protected void save() {
    SimpleDateFormat df = new SimpleDateFormat(Data.dateFormat);
    String name = visit.getPacient().getHashName() + "_" + df.format(new Date()) +
        ".zip";
    if (visit.getTestList().get(getType()).trim()) {
      tabletData.trim();
    }
    if (tabletData.save(new File(engine.getConfiguration().getDatabaseRoot(),
                                 visit.getPacient().getHashName() +
                                 File.separator + name))) {
      // make preparations for next capturing
      visit.addData(new Data(visit, tabletInfo, getType(), name, memo.getText()));
      visit.save();
      this.colorize();
      startButton.setText("Start");
      this.setDataEditEnabled(false);
      this.tabletData = new TabletData(tabletInfo);
      viewer.setTabletData(this.tabletData);
      screen.repaint();
      this.setRadioEnabled(true);
      data = false;
    }
  }

  protected void setDataEditEnabled(boolean enabled) {
    saveButton.setEnabled(enabled);
    memo.setEnabled(enabled);
    detailButton.setEnabled(enabled);
  }

  /**
   * called at beggining and end of capturing.
   */
  protected void setButtonsEnabled(boolean enabled) {
    exitButton.setEnabled(enabled);
    if (!enabled) {
      this.setRadioEnabled(enabled);
    }
  }

  /**
   * enables/disables types of drawings
   */
  private void setRadioEnabled(boolean enabled) {
    Enumeration e = type.getElements();
    while (e.hasMoreElements()) {
      ( (AbstractButton) e.nextElement()).setEnabled(enabled);
    }
  }

  /**
   * based on detail, set color of border of viewing area
   */
  public void setScreenBorder() {
    if (viewer.getDetail()) {
      screen.setBorder(BorderFactory.createEtchedBorder(Color.RED, Color.PINK));
    }
    else {
      screen.setBorder(BorderFactory.createEtchedBorder(Color.BLUE, Color.CYAN));
    }
  }
}

/**
 * <p>Button performing starting and ending of capturing data for CaptureFrame.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 *
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.3
 */
class Capture_Start_Button
    extends JButton implements ActionListener {
  /**
   * engine
   */
  Engine engine;
  /**
   * parent CaptureFrame
   */
  CaptureFrame adaptee;

  /**
   * create button for CaptureFrame
   */
  Capture_Start_Button(Engine engine, CaptureFrame adaptee) {
    super("Start");
    this.engine = engine;
    this.adaptee = adaptee;
    this.addActionListener(this);
    this.setMnemonic(KeyEvent.VK_S);
  }

  /**
   * called when start/stop button is pressed
   */
  public void actionPerformed(ActionEvent actionEvent) {
    if (!this.getText().equals("Stop")) { // start capturing
      this.setText("Stop");
      adaptee.startCapture();
    }
    else { // end capturing
      this.setText("Znovu");
      engine.getTablet().stopCapture();
      adaptee.setButtonsEnabled(true);
      adaptee.time.end();
      if (adaptee.data) {
        adaptee.setDataEditEnabled(true);
      }
    }
  }
}

/**
 * <p>Action called when button for saving captured data is clicked.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 *
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.1
 */
class Capture_Save_Action
    extends AbstractAction {
  /**
   * parent CaptureFrame
   */
  CaptureFrame adaptee;

  /**
   * create action for CaptureFrame
   */
  Capture_Save_Action(CaptureFrame adaptee) {
    super("Data OK");
    putValue(SHORT_DESCRIPTION, "Ulo nasnman data");
    putValue(MNEMONIC_KEY, KeyEvent.VK_U);
    this.adaptee = adaptee;
  }

  /**
   * called when save button is clicked
   */
  public void actionPerformed(ActionEvent actionEvent) {
    adaptee.save();
  }
}

/**
 * <p>Action called when frame is being closed.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 *
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.1
 */
class Capture_Exit_Action
    extends AbstractAction implements WindowListener {
  /**
   * parent CaptureFrame
   */
  CaptureFrame adaptee;

  /**
   * create action for CaptureFrame
   */
  Capture_Exit_Action(CaptureFrame adaptee) {
    super("Ukoni");
    this.adaptee = adaptee;
  }

  /**
   * Need for sake of Java
   */
  public void windowActivated(WindowEvent e) {}

  /**
   * Need for sake of Java
   */
  public void windowClosed(WindowEvent e) {}

  /**
   * Need for sake of Java
   */
  public void windowDeactivated(WindowEvent e) {}

  /**
   * Need for sake of Java
   */
  public void windowDeiconified(WindowEvent e) {}

  /**
   * Need for sake of Java
   */
  public void windowIconified(WindowEvent e) {}

  /**
   * Need for sake of Java
   */
  public void windowOpened(WindowEvent e) {}

  /**
   * called when closing window
   */
  public void windowClosing(WindowEvent e) {
    this.exit();
  }

  /**
   * called when close button clicked
   */
  public void actionPerformed(ActionEvent e) {
    this.exit();
  }

  /**
   * try to close window and save data
   */
  private void exit() {
    if (adaptee.saveButton.isEnabled()) {
      Object[] options = {
          "Ano", "Ne", "Zruit"};
      int ret = JOptionPane.showOptionDialog(null,
                                             "Byla sejmuta nov data. Uloit?",
                                             "Nov data",
                                             JOptionPane.YES_NO_CANCEL_OPTION,
                                             JOptionPane.WARNING_MESSAGE, null,
                                             options, options[2]);
      switch (ret) {
        case JOptionPane.YES_OPTION:
          adaptee.save();
        case JOptionPane.NO_OPTION:
          adaptee.dispose();
      }
    }
    else if (!adaptee.startButton.getText().equals("Stop")) {
      adaptee.dispose();
    }
  }
};

class TimeThread
    extends Thread {
  private boolean running;
  private CaptureFrame frame;
  private BufferedImage img = null;
  private int width;
  private int height;
  private Font font = new Font("Tahoma", Font.PLAIN, 20);

  public TimeThread(CaptureFrame cf) {
    frame = cf;
    FontMetrics fm = cf.getFontMetrics(font);
    width = fm.stringWidth("8888");
    height = fm.getHeight();
    img = new BufferedImage(width, height, BufferedImage.TYPE_3BYTE_BGR);
  }

  public void run() {
    running = true;
    while (running) {
      long sec = (System.currentTimeMillis() - frame.startTime) / 1000;
      if (!frame.firstPacket) {
        long min = sec / 60;
        Graphics g = frame.screen.getGraphics();
        Graphics imgg = img.getGraphics();
        imgg.setColor(Color.WHITE);
        imgg.fillRect(0, 0 , width, height);
        imgg.setColor(Color.RED);
        imgg.setFont(font);
        imgg.drawString(String.format("%d:%02d", min, sec % 60), 0, height);
        g.drawImage(img, 6, 6, width, height, null);
      }
      try {
        sleep(1000);
      }
      catch (Exception e) {}
    }
  }

  public synchronized void end() {
    running = false;
  }
}
