/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-11-30
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
*/

package spirals.motol.gui;

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import spirals.data.*;
import spirals.motol.database.*;
import tablet.*;

/**
 * <p>Frame for editing captured data.</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.1
 */
public class EditDataFrame extends JDialog {
  /**
   * views captured data
   */
  private SimpleTabletDataViewer viewer;
  /**
   * contains content of frame
   */
  private JPanel contentPane;
  /**
   * performs clean exit
   */
  private ExitListener ea = new ExitListener(this);
  /**
   * visit which data belongs to
   */
  protected Visit visit;
  /**
   * data for editing
   */
  protected Data data;
  /**
   * data has been changed
   */
  protected boolean changed = false;
  /**
   * list of available types of drawings
   */
  protected JComboBox type;
  /**
   * list of available tablets
   */
  protected JComboBox tablet;
  /**
   * comment for data
   */
  protected JEditorPane memo = new JEditorPane();

  /**
   * area where is performed drawing
   */
  private JPanel screen = new JPanel() {
    public Dimension getPreferredSize() {
      return new Dimension(400,300);
    }

    public void paintComponent(Graphics g) {
      viewer.paint(g);
    }
  };

  /**
   * create edit frame for Data in Visit
   */
  public EditDataFrame(JFrame parent, Visit visit, Data data) {
    super(parent, data.toString(), true);
    this.visit = visit;
    this.data = data;
    Test t = visit.getTestList().get(data.getDescription());
    this.viewer = new SimpleTabletDataViewer(screen, new TabletData(data.getFile(), data.getTabletInfo()), t.showUp());
    this.contentPane = (JPanel) this.getContentPane();
    this.setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
    this.addWindowListener(ea);
    contentPane.setBorder(BorderFactory.createEmptyBorder(10,10,10,10));
    screen.setBorder(BorderFactory.createEtchedBorder());
    JPanel right = new JPanel(new BorderLayout());
    right.setBorder(BorderFactory.createEmptyBorder(0,10,10,0));

    JPanel top = new JPanel(new GridLayout(4,1));
    // type of drawing
    type = new JComboBox(visit.getTestList().getAll().toArray());
    type.setSelectedItem(data.getDescription());
    type.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        changed = true;
      }
    });
    top.add(new JLabel("Typ:"));
    top.add(type);

    // tablet
    tablet = new JComboBox(visit.getPacient().getDatabase().getEngine().getConfiguration().getAvailableTablets().toArray());
    tablet.setSelectedItem(data.getTabletInfo());
    tablet.addItemListener(new ItemListener() {
      public void itemStateChanged(ItemEvent e) {
        changed = true;
        viewer.setTabletInfo((TabletInfo)tablet.getSelectedItem());
        screen.repaint();
      }
    });
    top.add(new JLabel("Tablet:"));
    top.add(tablet);
    top.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));

    // memo
    JPanel m = new JPanel(new GridBagLayout());
    memo.setText(data.getMemo());
    memo.addKeyListener(new KeyAdapter() {
      public void keyPressed(KeyEvent e) {
        changed = true;
      }
    });
    m.setBorder(BorderFactory.createEmptyBorder(0,0,20,0));
    m.add(new JLabel("Poznmka:"), new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0, 0, 0, 10), 0, 0));
    m.add(new JScrollPane(memo), new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(0, 0, 0, 10), 0, 0));
    right.add(m, BorderLayout.CENTER);

    // close button
    right.add(new JButton(new AbstractAction("Zavi") {
      public void actionPerformed(ActionEvent e) {
        ea.windowClosing(null);
      }
    }), BorderLayout.SOUTH);
    right.add(top, BorderLayout.NORTH);
    this.add(screen, BorderLayout.CENTER);
    this.add(right, BorderLayout.EAST);
  }
}

/**
 * <p>Class performs clean exit from EditDataFrame</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 *
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.3
 */
class ExitListener extends WindowAdapter {
  /**
   * parent EditDataFrame
   */
  private EditDataFrame adaptee;

  /**
   * create listener for EditDataFrame
   */
  public ExitListener(EditDataFrame adaptee) {
    this.adaptee = adaptee;
  }

  /**
   * executed on window closing
   */
  public void windowClosing(WindowEvent e) {
    if(adaptee.changed) {
      Object[] options = {"Ano", "Ne"};
      int ret = JOptionPane.showOptionDialog(adaptee, "daje o datech zmnny. Uloit?", "Data zmnna", JOptionPane.YES_NO_OPTION, JOptionPane.WARNING_MESSAGE, null, options, options[0]);
      if(ret == JOptionPane.YES_OPTION) {
        adaptee.data.setDescription((String) adaptee.type.getSelectedItem());
        adaptee.data.setTabletInfo((TabletInfo) adaptee.tablet.getSelectedItem());
        adaptee.data.setMemo(adaptee.memo.getText());
        adaptee.visit.save();
      }
    }
    adaptee.dispose();
  }
}
