/*
 *	Author: Jan Dolezel, http://www.eithel.net
 *	Last version: http://www.eithel.net/download/ui/spiral
 *	Last modified: 2005-09-12
 *
 *	License Information:
 *	-------------------------------------------------------------------------
 *	Copyright (C) 2005 Jan Dolezel
 *
 *	This program is free software; you can redistribute it and/or modify it
 *	under the terms of the GNU General Public License as published by the
 *	Free Software Foundation; either version 2 of the License, or (at your
 *	option) any later version.
 *
 *	This program is distributed in the hope that it will be useful, but
 *	WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 *	General Public License for more details.
*/

package spirals.motol.gui;

import javax.swing.JPanel;
import java.awt.LayoutManager;

/**
 * <p>Abstract class for all removable panels for placing right part of main frame</p>
 *
 * <p>Copyright: Copyright (c) 2005 Jan Dolezel</p>
 * <p>Company: VUT FEL, Prague</p>
 *
 * @author Jan Dolezel
 * @version 1.1.1
 */
abstract public class RemovablePanel extends JPanel {
  /**
   * record changed by user
   */
  private boolean changed = false;

  /**
   * create RemovablePanel with LayoutManager
   */
  public RemovablePanel(LayoutManager l) {
    super(l);
  }

  /**
   * notify when panel is removed
   */
  public final void notifyRemoved() {
    if(changed) this.remove();
  }

  /**
   * user has changed record
   */
  protected void setChanged(boolean changed) {
    this.changed = changed;
  }

  /**
   * user has changed record
   */
  protected boolean hasChanged() {
    return changed;
  }

  /**
   * abstract method must override all descendants when they want to clean after themselves
   */
  abstract protected void remove();
}
